/*---------------------------------------------------------------------------
  Project:  ImageDb
  File:     JpegMpLoader.h
  
  JPEG/MP files are read from an SD Card and the decoded results are returned.
  ---------------------------------------------------------------------------*/
#ifndef JPEG_MP_LOADER_H_
#define JPEG_MP_LOADER_H_


#include <nn/types.h>
#include <nn/util.h>
#include <nn/jpeg.h>
#include <imgdb/Core/ImageDbResult.h>


namespace imgdb {


/*
  
 */
#pragma pack(push, 4)
class JpegMpBaseLoader
{
public:
    /*
      
      */
    JpegMpBaseLoader();

    /*
      
      */
    virtual ~JpegMpBaseLoader();


    /*=======================================================================
      Information to set before calling Load()
      =======================================================================*/
    /*
      
      
      */
    void*           GetUserMakerNoteBuffer() const;
    /*
      
      
      */
    std::size_t     GetUserMakerNoteBufferSize() const;
    /*
      
      
      
      
      
      */
    void    SetUserMakerNoteBuffer(void* pBuf, std::size_t nSize);


    /*=======================================================================
      Information that can be obtained after a successful call to Load()
      =======================================================================*/
    /*
      
      
      
      */
    bool    GetDateTime(nn::fnd::DateTime& rDateTime) const;

    /*-----------------------------------------------------------------------
      MakerNotes for applications
      -----------------------------------------------------------------------*/
    /*
      
      
      
      
      
      
      
      */
    std::size_t   GetUserMakerNoteDataSize() const;

    /*-----------------------------------------------------------------------
      System maker note
      -----------------------------------------------------------------------*/
    /*
      
      
      */
    bool    IsExistSysMakerNote() const;

    /*
      
      
      */
    static std::size_t  GetSysMakerNoteDataSize();
    /*
      
      
      */
    const void*   GetSysMakerNoteData() const;

    /*-----------------------------------------------------------------------
      The title's unique ID
      -----------------------------------------------------------------------*/
    /*
      
      
      
      
      */
    u32     GetTitleUniqueId() const;

    /*-----------------------------------------------------------------------
      Attribute
      -----------------------------------------------------------------------*/
    /*
      
      
      
      */
    bool    GetScreenshotFlag() const;
    /*
      
      
      */
    bool    GetAdjustablePictureFlag() const;

    /*
      
      
      */
    bool IsExistFace() const;

    /*-----------------------------------------------------------------------
      System ID
      -----------------------------------------------------------------------*/
    /*
      
      
      */
    bool    IsValidShotBodyId() const;
    /*
      
      
      
      
      */
    u32     GetShotBodyId() const;

    /*
      
      
      */
    bool    IsValidEditorBodyId() const;
    /*
      
      
      
      
      */
    u32     GetEditorBodyId() const;


    /*=======================================================================
      Other
      =======================================================================*/
    /*
      
      
      
      
      
      */
    static std::size_t ComputeImageSize(
                        int nWidth, int nHeight, nn::jpeg::CTR::PixelFormat eFormat);

protected:
    void    SetDateTime(const nn::fnd::DateTime& rDateTime);
    void    ResetDateTime();

    bool    SetSysMakerNoteData(const void* pNote, std::size_t nSize);

    void    SetUserMakerNoteDataSize(std::size_t nSize);

    void    SetScreenshotFlag(bool bFlag);
    void    SetAdjustablePictureFlag(bool bFlag);

    void    SetShotBodyId(u32 nBodyId);
    void    ResetShotBodyId();

    void    SetEditorBodyId(u32 nBodyId);
    void    ResetEditorBodyId();

private:
    bool    GetHandleFlag(int nHandleType) const;
    void    SetHandleFlag(int nHandleType, bool bFlag);

    bool    IsValidBodyId(int nIdType) const;
    u32     GetBodyId(int nIdType) const;
    void    SetBodyId(int nIdType, u32 nBodyId);
    void    ResetBodyId(int nIdType);

private:
    void*               m_pUserMakerNoteBuf;
    std::size_t         m_pUserMakerNoteBufSize;
    nn::fnd::DateTime   m_nDateTime;
    bool                m_bValidDateTime;
    NN_PADDING3;
    std::size_t         m_nUserMakerNoteDataSize;
    void*               m_pSysMakerNoteData;
};
#pragma pack(pop)


/*
  
  
  
   
   
   
   
 */
class JpegLoader : public JpegMpBaseLoader, private nn::util::NonCopyable<JpegLoader>
{
public:
    //
    static const nn::jpeg::CTR::PixelFormat c_eDefaultRawFormat = nn::jpeg::CTR::PIXEL_FORMAT_CTR_RGB565;
    //
    static const bool                       c_nDefaultThumbnailMode = false;
    //
    static const int                        c_nDefaultOutputWidth = 0;

public:
    /*
      
      */
    JpegLoader();

    /*
      
      */
    virtual ~JpegLoader();

    /*
      
      
      
      
      
      

      
      
      */
    Result  Load();


    /*=======================================================================
      Information to set before calling Load()
      =======================================================================*/
    /*
      
      
      */
    const wchar_t*  GetFilePath() const;
    /*
      
      
      
      
      
      */
    void    SetFilePath(const wchar_t* pPath);


    /*
      
      
      */
    nn::jpeg::CTR::PixelFormat  GetFormat() const;
    /*
      
      
      */
    void    SetFormat(nn::jpeg::CTR::PixelFormat eFormat);


    /*
      
      
      */
    int     GetOutputWidth();
    /*
      
      
      
      
      
      
      
      */
    void    SetOutputWidth(int nWidth);


    /*
      
      
      
      */
    bool    IsThumbnailMode() const;
    /*
      
      
      
      
      */
    void    SetThumbnailMode(bool bThumbMode);


    /*
      
      
      */
    void*           GetBuffer() const;
    /*
      
      
      */
    std::size_t     GetBufferSize() const;
    /*
      
      
      
      
      
      */
    void    SetBuffer(void* pBuf, std::size_t nSize);


    /*=======================================================================
      Information that can be obtained after a successful call to Load()
      =======================================================================*/
    /*
      
      
      
      
      */
    int     GetRawWidth() const;
    /*
      
      
      
      
      */
    int     GetRawHeight() const;
    /*
      
      
      */
    std::size_t GetRawDataSize() const;

    /*
      
      
      */
    int     GetRealWidth() const;
    /*
      
      
      */
    int     GetRealHeight() const;

private:
    void    ResetLoaded();
    bool    IsLoaded() const;

    Result  ValidateParam() const;

private:
    const wchar_t*                  m_pFilePath;
    int                             m_nOutputWidth;
    nn::jpeg::CTR::PixelFormat      m_eFormat;
    bool                            m_bThumbMode;
    NN_PADDING2;

    void*           m_pBuf;
    std::size_t     m_nBufSize;

    int             m_nRawWidth;
    int             m_nRawHeight;
    std::size_t     m_nRawDataSize;
    int             m_nRealWidth;
    int             m_nRealHeight;
};


/*
  
  
  
   
   
   
   
  
  
  
 */
class MpLoader : public JpegMpBaseLoader, private nn::util::NonCopyable<MpLoader>
{
public:
    //
    static const nn::jpeg::CTR::PixelFormat c_eDefaultRawFormat = nn::jpeg::CTR::PIXEL_FORMAT_CTR_RGB565;
    //
    static const bool                       c_nDefaultThumbnailMode = false;
    //
    static const int                        c_nDefaultOutputWidth = 0;

public:
    /*
      
      */
    MpLoader();
    /*
      
      */
    virtual ~MpLoader();


    /*
      
      
      
      
      
      

      
      
      */
    Result  Load();


    /*=======================================================================
      Information to set before calling Load()
      =======================================================================*/
    /*
      
      
      */
    const wchar_t*  GetFilePath() const;
    /*
      
      
      
      
      
      */
    void    SetFilePath(const wchar_t* pPath);


    /*
      
      
      */
    nn::jpeg::CTR::PixelFormat  GetFormat() const;
    /*
      
      
      */
    void    SetFormat(nn::jpeg::CTR::PixelFormat eFormat);


    /*
      
      
      */
    int     GetOutputWidth();
    /*
      
      
      
      
      
      
      
      */
    void    SetOutputWidth(int nWidth);


    /*
      
      
      
      */
    bool    IsThumbnailMode() const;
    /*
      
      
      
      
      */
    void    SetThumbnailMode(bool bThumbMode);


    /*
      
      
      */
    void*           GetBufferL();
    /*
      
      
      */
    void*           GetBufferR();
    /*
      
      
      */
    std::size_t     GetBufferSize() const;
    /*
      
      
      
      
      
      
      
      
      */
    void    SetBuffer(void* pBufL, void* pBufR, std::size_t nSize);


    /*=======================================================================
      Information that can be obtained after a successful call to Load()
      =======================================================================*/
    /*
      
      
      
      
      */
    int     GetRawWidth() const;
    /*
      
      
      
      
      */
    int     GetRawHeight() const;
    /*
      
      
      */
    std::size_t GetRawDataSize() const;

    /*
      
      
      */
    int     GetRealWidth() const;
    /*
      
      
      */
    int     GetRealHeight() const;

    /*
      
      
      
      
      
      
      
      
      
      
      */
    f32     GetOffsetH() const;

private:
    void    ResetLoaded();
    bool    IsLoaded() const;

    Result  ValidateParam() const;

private:
    const wchar_t*                  m_pFilePath;
    int                             m_nOutputWidth;
    nn::jpeg::CTR::PixelFormat      m_eFormat;
    bool                            m_bThumbMode;
    NN_PADDING2;

    void*           m_pBufL;
    void*           m_pBufR;
    std::size_t     m_nBufSize;

    int             m_nRawWidth;
    int             m_nRawHeight;
    std::size_t     m_nRawDataSize;
    int             m_nRealWidth;
    int             m_nRealHeight;
    f32             m_fOfstH;
};


} // end of namespace imgdb

#endif // JPEG_MP_LOADER_H_
