/*---------------------------------------------------------------------------
  Project:  ImageDb
  File:     JpegMpSaver.h
  
  When raw image data is received, it is converted into JPEG/MP and then saved to an SD Card.
  The saved images are then managed through Nintendo 3DS Camera.
  ---------------------------------------------------------------------------*/
#ifndef JPEG_MP_SAVER_H_
#define JPEG_MP_SAVER_H_

#include <nn/types.h>
#include <nn/util.h>
#include <nn/jpeg.h>
#include <imgdb/ImageIO/ImageIOTypes.h>
#include <imgdb/Core/ImageDbResult.h>


namespace imgdb {


/*
  
 */
class JpegMpBaseSaver
{
public:
    //
    static const bool               c_bDefaultScreenshotFlag = false;
    //
    static const bool               c_bDefaultAdjustablePictureFlag = false;
    //
    static const u32                c_nInvalidId = 0;
    //
    static const bool               c_bDefaultValidShotId = true;
    //
    static const bool               c_bDefaultValidEditorId = false;

public:
    /*
      
      */
    JpegMpBaseSaver();
    /*
      
      */
    virtual ~JpegMpBaseSaver();


    /*=======================================================================
      Information to set before calling Save()
      =======================================================================*/
    /*-----------------------------------------------------------------------
      MakerNotes for applications
      -----------------------------------------------------------------------*/
    /*
      
      
      
      
      */
    const void*   GetUserMakerNoteData() const;
    /*
      
      
      
      
      */
    std::size_t   GetUserMakerNoteDataSize() const;
    /*
      
      
      
      
      
      */
    void    SetUserMakerNote(const void* pData, std::size_t nSize);

    /*-----------------------------------------------------------------------
      System maker note
      -----------------------------------------------------------------------*/
    /*
      
      
      */
    static std::size_t  GetSysMakerNoteDataSize();
    /*
      
      
      */
    const void*   GetSysMakerNoteData() const;
    /*
      
      
      
      
      
      
      
      */
    bool    SetSysMakerNoteData(const void* pNote, std::size_t nSize);


    /*-----------------------------------------------------------------------
      The title's unique ID
      -----------------------------------------------------------------------*/
    /*
      
      
      
      
      */
    u32     GetTitleUniqueId() const;
    /*
      
      
      
      
      
      */
    void    SetTitleUniqueId(u32 nUId);

    /*-----------------------------------------------------------------------
      Attribute
      -----------------------------------------------------------------------*/
    /*
      
      
      
      */
    bool    GetScreenshotFlag() const;
    /*
      
      
      
      
      */
    void    SetScreenshotFlag(bool bFlag);

    /*
      
      
      */
    bool    GetAdjustablePictureFlag() const;
    /*
      
      
      
      
      
      
      
      
      
      */
    void    SetAdjustablePictureFlag(bool bFlag);

    /*-----------------------------------------------------------------------
      System ID
      -----------------------------------------------------------------------*/
    /*
      
      
      */
    bool    IsValidShotBodyId() const;
    /*
      
      
      
      
      
      */
    u32     GetShotBodyId() const;
    /*
      
      
      
      
      */
    void    SetShotBodyId(u32 nBodyId);
    /*
      
      
      
      
      */
    void    SetShotBodyIdByOwn();
    /*
      
      */
    void    ResetShotBodyId();

    /*
      
      
      */
    bool    IsValidEditorBodyId() const;
    /*
      
      
      
      
      */
    u32     GetEditorBodyId() const;
    /*
      
      
      
      
      
      
      */
    void    SetEditorBodyId(u32 nBodyId);
    /*
      
      
      
      
      */
    void    SetEditorBodyIdByOwn();
    /*
      
      */
    void    ResetEditorBodyId();

protected:
    bool    IsValidTitleUniqueId() const;

    bool    GetHandleFlag(int nHandleType) const;
    void    SetHandleFlag(int nHandleType, bool bFlag);

    bool    IsValidBodyId(int nIdType) const;
    u32     GetBodyId(int nIdType) const;
    void    SetBodyId(int nIdType, u32 nBodyId);
    void    ResetBodyId(int nIdType);

protected:
    const void* m_pUserMakerNoteData;
    std::size_t m_nUserMakerNoteDataSize;
    void*       m_pSysMakerNoteData;
    bool        m_bValidTitleUniqueId;
    NN_PADDING3;
};


/*
  
  
  
   
   
   
 */
class JpegSaver : public JpegMpBaseSaver, private nn::util::NonCopyable<JpegSaver>
{
public:
    //
    static const bool   c_bEnableQualityAutoAbatementFlag = true;
    //
    static const int                c_nDefaultJpegQuality = 80;
    //
    static const nn::jpeg::CTR::PixelSampling  c_eDefaultJpegFormat = nn::jpeg::CTR::PIXEL_SAMPLING_YUV422;

public:
    /*
      
      */
    JpegSaver();
    /*
      
      */
    virtual ~JpegSaver();

    /*
      
      
      
      
      
      

      
      
      */
    Result  Save();


    /*=======================================================================
      Information to set before calling Save()
      =======================================================================*/
    /*
      
      
      */
    bool    GetEnableQualityAutoAbatementFlag() const;
    /*
      
      
      
      
      
      
      
      */
    void    SetEnableQualityAutoAbatementFlag(bool bEnable);

    /*
      
      
      */
    int     GetJpegQuality() const;
    /*
      
      
      */
    void    SetJpegQuality(int nQuality);

    /*
      
      
      */
    nn::jpeg::CTR::PixelSampling   GetJpegFormat() const;
    /*
      
      
      */
    void    SetJpegFormat(nn::jpeg::CTR::PixelSampling eFormat);

    /*
      
      
      */
    const void* GetRawData() const;
    /*
      
      
      */
    int     GetRawWidth() const;
    /*
      
      
      */
    int     GetRawHeight() const;
    /*
      
      
      */
    nn::jpeg::CTR::PixelFormat  GetRawFormat() const;
    /*
      
      
      
      
      
      */
    void    SetRaw(
                const void* pData, int nWidth, int nHeight,
                nn::jpeg::CTR::PixelFormat eFormat);


    /*=======================================================================
      Information that can be obtained after a successful call to Save()
      =======================================================================*/
    /*
      
      
      
      
      */
    const wchar_t*  GetSavedFilePath() const;

private:
    Result  ValidateParam() const;
    void    SetFilePathToVirtual(const wchar_t* pPath);

private:
    int                             m_nJpegQuality;
    nn::jpeg::CTR::PixelSampling    m_eJpegFormat;
    bool                            m_bEnableQualityAutoAbatementFlag;
    NN_PADDING2;

    const void*                 m_pRawData;
    int                         m_nRawWidth;
    int                         m_nRawHeight;
    nn::jpeg::CTR::PixelFormat  m_eRawFormat;
    NN_PADDING3;
    wchar_t*                    m_pFilePath;
};


/*
  
  
  
   
   
   
 */
class MpSaver : public JpegMpBaseSaver, private nn::util::NonCopyable<MpSaver>
{
public:
    //
    static const bool   c_bEnableQualityAutoAbatementFlag = true;
    //
    static const int                c_nDefaultJpegQuality = 80;
    //
    static const nn::jpeg::CTR::PixelSampling  c_eDefaultJpegFormat = nn::jpeg::CTR::PIXEL_SAMPLING_YUV422;

public:
    /*
      
      */
    MpSaver();
    /*
      
      */
    virtual ~MpSaver();

    /*
      
      
      
      
      
      
      
      

      
      
      
      
      */
    Result  Save();


    /*=======================================================================
      Information to set before calling Save()
      =======================================================================*/
    /*
      
      
      */
    bool    GetEnableQualityAutoAbatementFlag() const;
    /*
      
      
      
      
      
      
      
      
      */
    void    SetEnableQualityAutoAbatementFlag(bool bEnable);

    /*
      
      
      */
    int     GetJpegQuality() const;
    /*
      
      
      */
    void    SetJpegQuality(int nQuality);

    /*
      
      
      */
    nn::jpeg::CTR::PixelSampling   GetJpegFormat() const;
    /*
      
      
      */
    void    SetJpegFormat(nn::jpeg::CTR::PixelSampling eFormat);

    /*
      
      
      */
    const void* GetRawDataL() const;
    /*
      
      
      */
    const void* GetRawDataR() const;
    /*
      
      
      */
    int     GetRawWidth() const;
    /*
      
      
      */
    int     GetRawHeight() const;
    /*
      
      
      */
    nn::jpeg::CTR::PixelFormat  GetRawFormat() const;
    /*
      
      
      
      
      
      
      */
    void    SetRaw(
                const void* pDataL, const void* pDataR,
                int nWidth, int nHeight, nn::jpeg::CTR::PixelFormat eFormat);

    /*
      
      
      */
    f32     GetOffsetH() const;
    /*
      
      
      */
    void    SetOffsetH(f32 fOfstH);


    /*=======================================================================
      Information that can be obtained after a successful call to Save()
      =======================================================================*/
    /*
      
      
      
      
      */
    const wchar_t*  GetSavedFilePath() const;

private:
    Result  ValidateParam() const;
    void    SetFilePathToVirtual(const wchar_t* pPath);

private:
    int                             m_nJpegQuality;
    nn::jpeg::CTR::PixelSampling    m_eJpegFormat;
    bool                            m_bEnableQualityAutoAbatementFlag;
    NN_PADDING2;

    const void*                 m_pRawDataL;
    const void*                 m_pRawDataR;
    int                         m_nRawWidth;
    int                         m_nRawHeight;
    nn::jpeg::CTR::PixelFormat  m_eRawFormat;
    NN_PADDING3;
    f32                         m_fOfstH;
    wchar_t*                    m_pFilePath;
};



} // end of namespace imgdb

#endif // JPEG_MP_SAVER_H_
