﻿#include <nn.h>
#include <nn/fnd.h>
#include <nn/cfg.h>
#include <nn/jpeg.h>

#include <imgdb/ImageIO.h>


namespace {


// Main heap size
const std::size_t   MAIN_MEMORY_SIZE = 2*1024*1024;

// Image to work with
const int IMAGE_WIDTH  = 640;
const int IMAGE_HEIGHT = 480;
const nn::jpeg::CTR::PixelFormat IMAGE_FORMAT = nn::jpeg::CTR::PIXEL_FORMAT_CTR_RGB565;

// Specify the 20-bit unique ID assigned to each title.
const u32 TITLE_UNIQUE_ID = 0xFF000;

/*---------------------------------------------------------------------------*
  Allocator
 *---------------------------------------------------------------------------*/
class MyAllocator : public nn::fnd::IAllocator
{
public:
    void    Initialize(){ m_Heap.Initialize(nn::os::GetHeapAddress(), nn::os::GetHeapSize()); }
    void    Finalize(){ m_Heap.Finalize(); }

    virtual void* Allocate(size_t size, s32 alignment){ return m_Heap.Allocate(size, alignment); }
    virtual void Free(void* memory){ m_Heap.Free(memory); }

private:
    nn::fnd::ExpHeap    m_Heap;
};

MyAllocator s_Allocator;

/*---------------------------------------------------------------------------*
  Processing to load and save JPEGs
 *---------------------------------------------------------------------------*/
void TestSaveLoad()
{
    imgdb::JpegSaver    jpgSaver;
    imgdb::JpegLoader   jpgLoader;

    // Create some image and save it as a JPEG
    {
        // Create an image
        void* pImageData = 0;
        const std::size_t nImageSize = imgdb::JpegLoader::ComputeImageSize(
                                            IMAGE_WIDTH, IMAGE_HEIGHT, IMAGE_FORMAT);
        pImageData = s_Allocator.Allocate(nImageSize, 32);
        std::memset(pImageData, 0xff, nImageSize); // This image is completely white

        // Configure the various settings for saving a JPEG
        {
            //---------- Required settings ----------//
            // Image to save
            jpgSaver.SetRaw(pImageData, IMAGE_WIDTH, IMAGE_HEIGHT, IMAGE_FORMAT);
            // The title's unique ID
            jpgSaver.SetTitleUniqueId(TITLE_UNIQUE_ID);

            //---------- Optional settings ----------//
            // User MakerNote
            const char* pUserMakerNoteData = "This is UserMakerNote!";
            jpgSaver.SetUserMakerNote(pUserMakerNoteData, strlen(pUserMakerNoteData)+1);
        }

        // Save
        {
            const imgdb::Result eResult = jpgSaver.Save();
            if(imgdb::ResultSuccess == eResult){
                NN_LOG("Saved! %ls\n", jpgSaver.GetSavedFilePath());
            }
            else if(imgdb::ResultErrorNotExistStorage == eResult ||
                    imgdb::ResultErrorWriteProtect == eResult ||
                    imgdb::ResultErrorInsufficientStorageSpace == eResult ||
                    imgdb::ResultErrorFileOperationFailure == eResult)
            {
                // Errors that must be handled
                NN_PANIC("Error! Result=%d\n", eResult);
            }
            else{
                NN_PANIC("Error! Result=%d\n", eResult);
            }
        }

        s_Allocator.Free(pImageData);
    }

    // Load the image that we just saved
    {
        const std::size_t nUserMakerNoteBufSize = 32;
        char  nUserMakerNoteBuf[nUserMakerNoteBufSize];

        // Create a buffer to load the image
        void* pImageData = 0;
        const std::size_t nImageSize = imgdb::JpegLoader::ComputeImageSize(
                                            IMAGE_WIDTH, IMAGE_HEIGHT, IMAGE_FORMAT);
        pImageData = s_Allocator.Allocate(nImageSize, 32);
        std::memset(pImageData, 0x00, nImageSize);

        // Configure the various settings for loading a JPEG
        {
            //---------- Required settings ----------//
            jpgLoader.SetFilePath(jpgSaver.GetSavedFilePath());
            jpgLoader.SetBuffer(pImageData, nImageSize);
            jpgLoader.SetFormat(IMAGE_FORMAT);

            //---------- Optional settings ----------//
            jpgLoader.SetUserMakerNoteBuffer(nUserMakerNoteBuf, nUserMakerNoteBufSize);
        }

        // Read
        {
            const imgdb::Result eResult = jpgLoader.Load();
            if(imgdb::ResultSuccess == eResult){
                NN_LOG("Loaded!\n");
            }
            else if(imgdb::ResultErrorNotExistStorage == eResult ||
                    imgdb::ResultErrorWriteProtect == eResult ||
                    imgdb::ResultErrorInsufficientStorageSpace == eResult ||
                    imgdb::ResultErrorFileOperationFailure == eResult)
            {
                // Errors that must be handled
                NN_PANIC("Error! Result=%d\n", eResult);
            }
            else{
                NN_PANIC("Error! Result=%d\n", eResult);
            }
        }

        // Extract the information that was set
        {
            NN_LOG(" TitleUniqueId              : 0x%08x\n", jpgLoader.GetTitleUniqueId());

            NN_LOG(" ScreenshotFlag             : %s\n", jpgLoader.GetScreenshotFlag() ? "true" : "false");
            NN_LOG(" AdjustablePictureFlag      : %s\n", jpgLoader.GetAdjustablePictureFlag() ? "true" : "false");

            if(jpgLoader.IsValidShotBodyId()){
                NN_LOG(" ShotBodyId                 : 0x%08x\n", jpgLoader.GetShotBodyId());
            }
            else{
                NN_LOG(" ShotBodyId                 : Invalid\n");
            }
            if(jpgLoader.IsValidEditorBodyId()){
                NN_LOG(" EditorBodyId               : 0x%08x\n", jpgLoader.GetEditorBodyId());
            }
            else{
                NN_LOG(" EditorBodyId               : Invalid\n");
            }

            NN_LOG(" UserMakerNote              : %dByte\n", jpgLoader.GetUserMakerNoteDataSize());
            if(0 < jpgLoader.GetUserMakerNoteDataSize()){
                nUserMakerNoteBuf[nUserMakerNoteBufSize-1] = '\0';
                NN_LOG( " \"%s\"\n", nUserMakerNoteBuf);
            }
        }

        s_Allocator.Free(pImageData);
    }

    // Enumerate images on the SD Card
    {
        NN_LOG("\n");
        imgdb::search::SearchPicture();
        for(int i = 0; i < imgdb::search::GetResultNum(); ++i){
            static const char* pImgKindStr[imgdb::imgio::ImageKindNum+1] = { "JPG", "MPO", "???", };

            // Get the title's unique ID.
            const u32 nId = imgdb::search::GetTitleUniqueId(i);

            // Get the path.
            wchar_t cPath[imgdb::imgio::c_nMaxPathSize];
            imgdb::search::GetPath(cPath, sizeof(cPath)/sizeof(cPath[0]), i);

            // Get the image type.
            const imgdb::imgio::ImageKind eKind = imgdb::search::GetImageKind(i);

            // Get the capture date and time.
            const nn::fnd::DateTimeParameters dtp = imgdb::search::GetDateTime(i).GetParameters();

            NN_LOG(
                "0x%05x \"%ls\" [%s] %04d/%02d/%02d %02d:%02d.%02d\n",
                nId, cPath, pImgKindStr[eKind], dtp.year, dtp.month, dtp.day, dtp.hour, dtp.minute, dtp.second);
        }
        NN_LOG("\n");
    }

    // Delete the saved photo
    {
        const imgdb::Result eResult =
                imgdb::util::DeletePicture(jpgSaver.GetSavedFilePath(), TITLE_UNIQUE_ID);
        if(imgdb::ResultSuccess == eResult){
            NN_LOG("Deleted! %ls\n", jpgSaver.GetSavedFilePath());
        }
        else if(imgdb::ResultErrorNotExistStorage == eResult ||
                imgdb::ResultErrorWriteProtect == eResult ||
                imgdb::ResultErrorInsufficientStorageSpace == eResult ||
                imgdb::ResultErrorFileOperationFailure == eResult)
        {
            // Errors that must be handled
            NN_PANIC("Error! Result=%d\n", eResult);
        }
        else{
            NN_PANIC("Error! Result=%d\n", eResult);
        }
    }
}

} // Namespace

//----------------------------------------------------------------
extern "C" void nninitStartUp()
{
}

/*---------------------------------------------------------------------------*
  Main processing
 *---------------------------------------------------------------------------*/
extern "C" void nnMain()
{
	nn::cfg::Initialize(); // We need to initialize the nn::cfg library because its features are used by the ImageDB library
	nn::fs::Initialize();

    nn::os::SetHeapSize(MAIN_MEMORY_SIZE);
    s_Allocator.Initialize();

	NN_LOG("start\n");
    {
        imgdb::Initialize(s_Allocator);
        imgdb::search::Initialize(); // Initlization for enumerating images.

        TestSaveLoad();

        imgdb::search::Finalize();
        imgdb::Finalize();
    }
	NN_LOG("end\n");

    s_Allocator.Finalize();

	nn::cfg::Finalize();
}
